@@ -16,7 +16,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '1.3.4';
+    $VERSION = '1.3.5';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
@@ -98,7 +98,7 @@ sub add_defaults {
         'pdf compression'   => 'auto',
         'quality'           => 75,
         'pages to scan'     => 1,
-        'unpaper on scan'   => TRUE,
+        'unpaper on scan'   => FALSE,
         'OCR on scan'       => TRUE,
         'frontend'          => 'libsane-perl',
         'rotate facing'     => 0,
