@@ -5,6 +5,7 @@ use strict;
 use Gtk2;
 use Glib 1.220 qw(TRUE FALSE);    # To get TRUE and FALSE
 use Gtk2::Gdk::Keysyms;
+use Data::Dumper;
 
 use Glib::Object::Subclass Gtk2::Window::,
   signals => {
@@ -37,7 +38,8 @@ use Glib::Object::Subclass Gtk2::Window::,
     ),
   ];
 
-our $VERSION = '1.3.4';
+our $VERSION = '1.3.5';
+my $EMPTY = q{};
 
 sub INIT_INSTANCE {
     my $self = shift;
@@ -88,6 +90,15 @@ sub on_key_press_event {
     return Gtk2::EVENT_STOP;
 }
 
+sub dump_or_stringify {
+    my ($val) = @_;
+    return (
+        defined $val
+        ? ( ref($val) eq $EMPTY ? $val : Dumper($val) )
+        : 'undef'
+    );
+}
+
 1;
 
 __END__
