@@ -7,7 +7,6 @@ use Glib qw(TRUE FALSE);   # To get TRUE and FALSE
 use Sane 0.05;             # To get SANE_NAME_PAGE_WIDTH & SANE_NAME_PAGE_HEIGHT
 use Gscan2pdf::Dialog;
 use feature 'switch';
-use Data::Dumper;
 my (
     $_MAX_PAGES,        $_MAX_INCREMENT, $_DOUBLE_INCREMENT,
     $_CANVAS_SIZE,      $_CANVAS_BORDER, $_CANVAS_POINT_SIZE,
@@ -19,6 +18,8 @@ BEGIN {
     use Gscan2pdf::Scanner::Options;
     Glib::Type->register_enum( 'Gscan2pdf::Scanner::Dialog::Side',
         qw(facing reverse) );
+    Glib::Type->register_enum( 'Gscan2pdf::Scanner::Dialog::Sided',
+        qw(single double) );
     use Readonly;
     Readonly $_MAX_PAGES         => 9999;
     Readonly $_MAX_INCREMENT     => 99;
@@ -34,10 +35,10 @@ BEGIN {
 # from http://gtk2-perl.sourceforge.net/doc/subclassing_widgets_in_perl.html
 use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     'new-scan' => {
-        param_types => ['Glib::UInt'],    # page number
+        param_types => [ 'Glib::String', 'Glib::UInt' ], # filename, page number
     },
     'changed-device' => {
-        param_types => ['Glib::String'],    # device name
+        param_types => ['Glib::String'],                 # device name
     },
     'changed-device-list' => {
         param_types => ['Glib::Scalar'],    # array of hashes with device info
@@ -178,20 +179,28 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
         [qw/readable writable/]    # flags
     ),
     Glib::ParamSpec->enum(
-        'side-to-scan',                        # name
-        'Side to scan',                        # nickname
-        'Either facing or reverse',            # blurb
-        'Gscan2pdf::Scanner::Dialog::Side',    # type
-        'facing',                              # default
-        [qw/readable writable/]                # flags
+        'sided',                                # name
+        'Sided',                                # nickname
+        'Either single or double',              # blurb
+        'Gscan2pdf::Scanner::Dialog::Sided',    # type
+        'single',                               # default
+        [qw/readable writable/]                 # flags
+    ),
+    Glib::ParamSpec->enum(
+        'side-to-scan',                         # name
+        'Side to scan',                         # nickname
+        'Either facing or reverse',             # blurb
+        'Gscan2pdf::Scanner::Dialog::Side',     # type
+        'facing',                               # default
+        [qw/readable writable/]                 # flags
     ),
     Glib::ParamSpec->object(
-        'available-scan-options',              # name
-        'Scan options available',              # nickname
+        'available-scan-options',               # name
+        'Scan options available',               # nickname
         'Scan options currently available, whether active, selected, or not'
-        ,                                      # blurb
-        'Gscan2pdf::Scanner::Options',         # package
-        [qw/readable writable/]                # flags
+        ,                                       # blurb
+        'Gscan2pdf::Scanner::Options',          # package
+        [qw/readable writable/]                 # flags
     ),
     Glib::ParamSpec->scalar(
         'current-scan-options',                               # name
@@ -216,7 +225,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '1.3.4';
+our $VERSION = '1.3.5';
 
 my ( $d, $d_sane, $logger, $tooltips );
 my $tolerance = 1;
@@ -403,20 +412,24 @@ sub INIT_INSTANCE {
     $frames->add($vboxs);
 
     # Single sided button
-    my $buttons = Gtk2::RadioButton->new( undef, $d->get('Single sided') );
-    $tooltips->set_tip( $buttons, $d->get('Source document is single-sided') );
-    $vboxs->pack_start( $buttons, TRUE, TRUE, 0 );
-    $buttons->signal_connect(
+    $self->{buttons} = Gtk2::RadioButton->new( undef, $d->get('Single sided') );
+    $tooltips->set_tip( $self->{buttons},
+        $d->get('Source document is single-sided') );
+    $vboxs->pack_start( $self->{buttons}, TRUE, TRUE, 0 );
+    $self->{buttons}->signal_connect(
         clicked => sub {
             $spin_buttoni->set_value(1);
+            $self->set( 'sided',
+                $self->{buttons}->get_active == 1 ? 'single' : 'double' );
         }
     );
 
     # Double sided button
-    my $buttond =
-      Gtk2::RadioButton->new( $buttons->get_group, $d->get('Double sided') );
-    $tooltips->set_tip( $buttond, $d->get('Source document is double-sided') );
-    $vboxs->pack_start( $buttond, FALSE, FALSE, 0 );
+    $self->{buttond} = Gtk2::RadioButton->new( $self->{buttons}->get_group,
+        $d->get('Double sided') );
+    $tooltips->set_tip( $self->{buttond},
+        $d->get('Source document is double-sided') );
+    $vboxs->pack_start( $self->{buttond}, FALSE, FALSE, 0 );
 
     # Facing/reverse page button
     my $hboxs = Gtk2::HBox->new;
@@ -424,15 +437,15 @@ sub INIT_INSTANCE {
     my $labels = Gtk2::Label->new( $d->get('Side to scan') );
     $hboxs->pack_start( $labels, FALSE, FALSE, 0 );
 
-    my $combobs = Gtk2::ComboBox->new_text;
+    $self->{combobs} = Gtk2::ComboBox->new_text;
     for ( ( $d->get('Facing'), $d->get('Reverse') ) ) {
-        $combobs->append_text($_);
+        $self->{combobs}->append_text($_);
     }
-    $combobs->signal_connect(
+    $self->{combobs}->signal_connect(
         changed => sub {
-            $buttond->set_active(TRUE);    # Set the radiobutton active
+            $self->{buttond}->set_active(TRUE);    # Set the radiobutton active
             $self->set( 'side-to-scan',
-                $combobs->get_active == 0 ? 'facing' : 'reverse' );
+                $self->{combobs}->get_active == 0 ? 'facing' : 'reverse' );
         }
     );
     $self->signal_connect(
@@ -442,19 +455,19 @@ sub INIT_INSTANCE {
                 $value eq 'facing' ? $_DOUBLE_INCREMENT : -$_DOUBLE_INCREMENT );
         }
     );
-    $tooltips->set_tip( $combobs,
+    $tooltips->set_tip( $self->{combobs},
         $d->get('Sets which side of a double-sided document is scanned') );
-    $combobs->set_active(0);
+    $self->{combobs}->set_active(0);
 
     # Have to do this here because setting the facing combobox switches it
-    $buttons->set_active(TRUE);
-    $hboxs->pack_end( $combobs, FALSE, FALSE, 0 );
+    $self->{buttons}->set_active(TRUE);
+    $hboxs->pack_end( $self->{combobs}, FALSE, FALSE, 0 );
 
     # Have to put the double-sided callback here to reference page side
-    $buttond->signal_connect(
+    $self->{buttond}->signal_connect(
         clicked => sub {
             $spin_buttoni->set_value(
-                  $combobs->get_active == 0
+                  $self->{combobs}->get_active == 0
                 ? $_DOUBLE_INCREMENT
                 : -$_DOUBLE_INCREMENT
             );
@@ -464,7 +477,7 @@ sub INIT_INSTANCE {
 # Have to put the extended pagenumber checkbox here to reference simple controls
     $checkx->signal_connect(
         toggled => \&_extended_pagenumber_checkbox_callback,
-        [ $self, $frames, $spin_buttoni, $buttons, $buttond, $combobs ]
+        [ $self, $frames, $spin_buttoni ]
     );
 
     # Scan profiles
@@ -582,29 +595,13 @@ sub SET_PROPERTY {
     elsif (( defined $newval and defined $oldval and $newval ne $oldval )
         or ( defined $newval xor defined $oldval ) )
     {
+        my $msg;
         if ( defined $logger ) {
-            $logger->debug(
-                "Started setting $name from "
-                  . (
-                    defined $oldval
-                    ? (
-                        ref($oldval) =~ /(?:HASH|ARRAY)/xsm
-                        ? Dumper($oldval)
-                        : $oldval
-                      )
-                    : 'undef'
-                  )
-                  . ' to '
-                  . (
-                    defined $newval
-                    ? (
-                        ref($newval) =~ /(?:HASH|ARRAY)/xsm
-                        ? Dumper($newval)
-                        : $newval
-                      )
-                    : 'undef'
-                  )
-            );
+            $msg =
+                " setting $name from "
+              . Gscan2pdf::Dialog::dump_or_stringify($oldval) . ' to '
+              . Gscan2pdf::Dialog::dump_or_stringify($newval);
+            $logger->debug("Started$msg");
         }
         given ($name) {
             when ('device') {
@@ -625,7 +622,19 @@ sub SET_PROPERTY {
                 $self->signal_emit( 'changed-page-number-increment', $newval )
             }
             when ('side_to_scan') {
-                $self->signal_emit( 'changed-side-to-scan', $newval )
+                $self->signal_emit( 'changed-side-to-scan', $newval );
+                $self->{combobs}->set_active( $newval eq 'facing' ? 0 : 1 );
+            }
+            when ('sided') {
+                my $widget = $self->{buttons};
+                if ( $newval eq 'double' ) {
+                    $widget = $self->{buttond};
+                }
+                else {
+                    # selecting single-sided also selects facing page.
+                    $self->set( 'side-to-scan', 'facing' );
+                }
+                $widget->set_active(TRUE);
             }
             when ('paper') {
                 if ( set_combobox_by_text( $self->{combobp}, $newval ) ) {
@@ -640,8 +649,9 @@ sub SET_PROPERTY {
                 $self->set_profile($newval);
             }
 
-   # This resets all options, so also clear the profile and current-scan-options
-   # options, but without setting off their signals
+            # This resets all options, so also clear the profile and
+            # current-scan-options options, but without setting off their
+            # signals
             when ('available_scan_options') {
                 $self->{profile}              = undef;
                 $self->{current_scan_options} = undef;
@@ -658,30 +668,7 @@ sub SET_PROPERTY {
                 $self->SUPER::SET_PROPERTY( $pspec, $newval );
             }
         }
-        if ( defined $logger ) {
-            $logger->debug(
-                "Finished setting $name from "
-                  . (
-                    defined $oldval
-                    ? (
-                        ref($oldval) =~ /(?:HASH|ARRAY)/xsm
-                        ? Dumper($oldval)
-                        : $oldval
-                      )
-                    : 'undef'
-                  )
-                  . ' to '
-                  . (
-                    defined $newval
-                    ? (
-                        ref($newval) =~ /(?:HASH|ARRAY)/xsm
-                        ? Dumper($newval)
-                        : $newval
-                      )
-                    : 'undef'
-                  )
-            );
-        }
+        if ( defined $logger ) { $logger->debug("Finished$msg") }
     }
     return;
 }
@@ -1123,23 +1110,22 @@ sub set_combobox_by_text {
 
 sub _extended_pagenumber_checkbox_callback {
     my ( $widget, $data ) = @_;
-    my ( $dialog, $frames, $spin_buttoni, $buttons, $buttond, $combobs ) =
-      @{$data};
+    my ( $dialog, $frames, $spin_buttoni ) = @{$data};
     if ( $widget->get_active ) {
         $frames->hide_all;
         $dialog->{framex}->show_all;
     }
     else {
         if ( $spin_buttoni->get_value == 1 ) {
-            $buttons->set_active(TRUE);
+            $widget->{buttons}->set_active(TRUE);
         }
         elsif ( $spin_buttoni->get_value > 0 ) {
-            $buttond->set_active(TRUE);
-            $combobs->set_active(0);
+            $widget->{buttond}->set_active(TRUE);
+            $widget->{combobs}->set_active(0);
         }
         else {
-            $buttond->set_active(TRUE);
-            $combobs->set_active(1);
+            $widget->{buttond}->set_active(TRUE);
+            $widget->{combobs}->set_active(1);
         }
         $frames->show_all;
         $dialog->{framex}->hide_all;
@@ -1479,21 +1465,115 @@ sub my_dumper {
     return;
 }
 
-# Helper sub to reduce code duplication
+# Set options to profile referenced by hashref
 
-sub set_option_emit_signal {
-    my ( $self, $i, $defaults, $signal1, $signal2 ) = @_;
-    if ( $i < @{$defaults} ) {
-        $i = $self->set_option_widget( $i, $defaults );
+sub set_current_scan_options {
+    my ( $self, $profile ) = @_;
+
+    if ( not defined $profile ) { return }
+
+    # As scanimage and scanadf rename the geometry options,
+    # we have to map them back to the original names
+    $self->map_geometry_names($profile);
+
+    # Move them first to a dummy array, as otherwise it would be self-modifying
+    my @defaults;
+
+    # Config::General flattens arrays with 1 entry to scalars,
+    # so we must check for this
+    if ( ref($profile) ne 'ARRAY' ) {
+        push @defaults, $profile;
+    }
+    else {
+        @defaults = @{$profile};
     }
 
-    # Only emit the changed-current-scan-options signal when we have finished
-    if (    ( not defined $i or $i > $#{$defaults} )
-        and $self->signal_handler_is_connected($signal1)
-        and $self->signal_handler_is_connected($signal2) )
-    {
-        $self->signal_handler_disconnect($signal1);
-        $self->signal_handler_disconnect($signal2);
+    # Give the GUI a chance to catch up between settings,
+    # in case they have to be reloaded.
+    # Use the callback to trigger the next loop
+    $self->_set_option_profile( 0, \@defaults );
+    return;
+}
+
+sub _set_option_profile {
+    my ( $self, $i, $profile ) = @_;
+    if ( $i < @{$profile} ) {
+
+        # for reasons I don't understand, without walking the reference tree,
+        # parts of $profile are undef
+        Gscan2pdf::Dialog::Scan::my_dumper( $profile->[$i] );
+        my ( $name, $val ) = each %{ $profile->[$i] };
+
+        if ( $name eq 'Paper size' ) {
+            $self->set( 'paper', $val );
+            $self->_set_option_profile( $i + 1, $profile );
+            return;
+        }
+
+        my $options = $self->get('available-scan-options');
+        my $opt     = $options->by_name($name);
+        if ( not defined $opt or $opt->{cap} & SANE_CAP_INACTIVE ) {
+            $logger->warn("Ignoring inactive option '$name'.");
+            splice @{$profile}, $i, 1;
+
+            # Update current_scan_options with a copy of the profile to prevent
+            # other callbacks changing it whilst we are applying it
+            my @dummy = @{$profile};
+            $self->{current_scan_options} = \@dummy;
+            $self->_set_option_profile( $i, $profile );
+            return;
+        }
+
+        $logger->debug("Setting option '$name' to '$val'.");
+        $self->set_option(
+            $opt, $val,
+            sub {
+                $self->_set_option_profile( $i + 1, $profile );
+            }
+        );
+
+        my $widget = $opt->{widget};
+        if ( defined $widget ) {
+            $logger->debug("Setting widget '$name' to '$val'.");
+            given ($widget) {
+                when ( $widget->isa('Gtk2::CheckButton') ) {
+                    if ( $val eq $EMPTY ) { $val = 0 }
+                    if ( $widget->get_active != $val ) {
+                        $widget->set_active($val);
+                        return $i;
+                    }
+                }
+                when ( $widget->isa('Gtk2::SpinButton') ) {
+                    if ( $widget->get_value != $val ) {
+                        $widget->set_value($val);
+                        return $i;
+                    }
+                }
+                when ( $widget->isa('Gtk2::ComboBox') ) {
+                    if ( $opt->{constraint}[ $widget->get_active ] ne $val ) {
+                        my $index;
+                        for ( 0 .. $#{ $opt->{constraint} } ) {
+                            if ( $opt->{constraint}[$_] eq $val ) {
+                                $index = $_;
+                            }
+                        }
+                        if ( defined $index ) { $widget->set_active($index) }
+                        return $i;
+                    }
+                }
+                when ( $widget->isa('Gtk2::Entry') ) {
+                    if ( $widget->get_text ne $val ) {
+                        $widget->set_text($val);
+                        return $i;
+                    }
+                }
+            }
+        }
+        else {
+            $logger->warn("Widget for option '$name' undefined.");
+        }
+    }
+    else {
         if ( not $self->{setting_profile} ) {
             $self->set( 'profile', undef );
         }
@@ -1502,7 +1582,7 @@ sub set_option_emit_signal {
             $self->get('current-scan-options')
         );
     }
-    return $i;
+    return;
 }
 
 # Extract a option value from a profile
