@@ -9,6 +9,8 @@ use Sane 0.05;             # To get SANE_NAME_PAGE_WIDTH & SANE_NAME_PAGE_HEIGHT
 use Gscan2pdf::Frontend::Sane;
 use Locale::gettext 1.05;    # For translations
 use feature 'switch';
+use Readonly;
+Readonly my $LAST_PAGE => -1;
 
 # logger duplicated from Gscan2pdf::Dialog::Scan
 # to ensure that SET_PROPERTIES gets called in both places
@@ -21,7 +23,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
     ),
 ];
 
-our $VERSION = '1.3.4';
+our $VERSION = '1.3.5';
 
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
@@ -29,6 +31,7 @@ my $SANE_NAME_SCAN_BR_X   = SANE_NAME_SCAN_BR_X;
 my $SANE_NAME_SCAN_BR_Y   = SANE_NAME_SCAN_BR_Y;
 my $SANE_NAME_PAGE_HEIGHT = SANE_NAME_PAGE_HEIGHT;
 my $SANE_NAME_PAGE_WIDTH  = SANE_NAME_PAGE_WIDTH;
+my $EMPTY                 = q{};
 my ( $d, $d_sane, $logger, $tooltips );
 
 sub INIT_INSTANCE {
@@ -108,9 +111,7 @@ sub scan_options {
 
     # Remove any existing pages
     while ( $self->{notebook}->get_n_pages > 1 ) {
-
-        # -1 = last page
-        $self->{notebook}->remove_page(-1);  ## no critic (ProhibitMagicNumbers)
+        $self->{notebook}->remove_page($LAST_PAGE);
     }
 
     # Ghost the scan button whilst options being updated
@@ -454,7 +455,7 @@ sub hide_geometry {
 # and walking the options tree, update the widgets
 
 sub set_option {
-    my ( $self, $option, $val ) = @_;
+    my ( $self, $option, $val, $finished_callback ) = @_;
     if ( not defined $option ) { return }
 
     $self->add_to_current_scan_options( $option, $val );
@@ -484,7 +485,8 @@ sub set_option {
             }
 
             # We can carry on applying defaults now, if necessary.
-            $self->signal_emit( 'finished-process', 'set_option' );
+            $self->signal_emit( 'finished-process',
+                "set_option $option->{name} to $val" );
 
             # Unset the profile unless we are actively setting it
             if ( not $self->{setting_profile} ) {
@@ -497,7 +499,13 @@ sub set_option {
             }
 
             $self->signal_emit( 'changed-scan-option', $option->{name}, $val );
-        }
+            if ( defined $finished_callback ) { $finished_callback->() }
+        },
+        error_callback => sub {
+            my ($message) = @_;
+            $self->signal_emit( 'process-error', 'set_option',
+                $d->get( 'Error setting option: ' . $message ) );
+        },
     );
     return;
 }
@@ -596,91 +604,16 @@ sub update_options {
     return;
 }
 
-# Set options to profile referenced by hashref
-
-sub set_current_scan_options {
+# As scanimage and scanadf rename the geometry options,
+# we have to map them back to the original names
+sub map_geometry_names {
     my ( $self, $profile ) = @_;
-
-    if ( not defined $profile ) { return }
-
-    # Move them first to a dummy array, as otherwise it would be self-modifying
-    my $defaults;
-
-    # Config::General flattens arrays with 1 entry to scalars,
-    # so we must check for this
-    if ( ref($profile) ne 'ARRAY' ) {
-        push @{$defaults}, $profile;
-    }
-    else {
-        @{$defaults} = @{$profile};
-    }
-
-    # Give the GUI a chance to catch up between settings,
-    # in case they have to be reloaded.
-    # Use the 'changed-scan-option' signal to trigger the next loop
-    my $i = 0;
-
-    my ( $changed_scan_signal, $changed_paper_signal );
-    $changed_scan_signal = $self->signal_connect(
-        'changed-scan-option' => sub {
-            my ( $widget, $name, $val ) = @_;
-
-           # for reasons I don't understand, without walking the reference tree,
-           # parts of $default are undef
-            Gscan2pdf::Dialog::Scan::my_dumper($defaults);
-            my ( $ename, $eval ) = each %{ $defaults->[$i] };
-
-            # don't check $eval against $val, just in case they are different
-            if ( $ename eq $name ) {
-                $i++;
-                $i =
-                  $self->set_option_emit_signal( $i, $defaults,
-                    $changed_scan_signal, $changed_paper_signal );
-            }
-        }
-    );
-    $changed_paper_signal = $self->signal_connect(
-        'changed-paper' => sub {
-            my ( $widget, $val ) = @_;
-
-           # for reasons I don't understand, without walking the reference tree,
-           # parts of $default are undef
-            Gscan2pdf::Dialog::Scan::my_dumper($defaults);
-            my ( $ename, $eval ) = each %{ $defaults->[$i] };
-
-            if ( $eval eq $val ) {
-                $i++;
-                $i =
-                  $self->set_option_emit_signal( $i, $defaults,
-                    $changed_scan_signal, $changed_paper_signal );
-            }
-        }
-    );
-    $i =
-      $self->set_option_emit_signal( $i, $defaults, $changed_scan_signal,
-        $changed_paper_signal );
-    return;
-}
-
-# Set option widget
-
-sub set_option_widget {
-    my ( $self, $i, $profile ) = @_;
-
-    while ( $i < @{$profile} ) {
+    for my $i ( 0 .. $#{$profile} ) {
 
         # for reasons I don't understand, without walking the reference tree,
         # parts of $profile are undef
         Gscan2pdf::Dialog::Scan::my_dumper( $profile->[$i] );
         my ( $name, $val ) = each %{ $profile->[$i] };
-
-        if ( $name eq 'Paper size' ) {
-            $self->set( 'paper', $val );
-            return $self->set_option_widget( $i + 1, $profile );
-        }
-
-        # As scanimage and scanadf rename the geometry options,
-        # we have to map them back to the original names
         given ($name) {
             when ('l') {
                 $name = SANE_NAME_SCAN_TL_X;
@@ -713,59 +646,7 @@ sub set_option_widget {
                 $profile->[$i] = { $name => $val };
             }
         }
-
-        my $options = $self->get('available-scan-options');
-        my $opt     = $options->by_name($name);
-        my $widget  = $opt->{widget};
-
-        if ( ref($val) eq 'ARRAY' ) {
-            $self->set_option( $opt, $val );
-
-            # when INFO_INEXACT is implemented, so that the value is reloaded,
-            # check for it here, so that the reloaded value is not overwritten.
-            $opt->{val} = $val;
-        }
-        elsif ( defined $widget ) {
-            given ($widget) {
-                when ( $widget->isa('Gtk2::CheckButton') ) {
-                    if ( $val eq '' ) { $val = 0 }
-                    if ( $widget->get_active != $val ) {
-                        $widget->set_active($val);
-                        return $i;
-                    }
-                }
-                when ( $widget->isa('Gtk2::SpinButton') ) {
-                    if ( $widget->get_value != $val ) {
-                        $widget->set_value($val);
-                        return $i;
-                    }
-                }
-                when ( $widget->isa('Gtk2::ComboBox') ) {
-                    if ( $opt->{constraint}[ $widget->get_active ] ne $val ) {
-                        my $index;
-                        for ( 0 .. $#{ $opt->{constraint} } ) {
-                            if ( $opt->{constraint}[$_] eq $val ) {
-                                $index = $_;
-                            }
-                        }
-                        if ( defined $index ) { $widget->set_active($index) }
-                        return $i;
-                    }
-                }
-                when ( $widget->isa('Gtk2::Entry') ) {
-                    if ( $widget->get_text ne $val ) {
-                        $widget->set_text($val);
-                        return $i;
-                    }
-                }
-            }
-        }
-        else {
-            $logger->warn("Widget for option '$name' undefined.");
-        }
-        ++$i;
     }
-
     return;
 }
 
@@ -787,7 +668,6 @@ sub scan {
     my $i = 1;
     Gscan2pdf::Frontend::Sane->scan_pages(
         dir              => $self->get('dir'),
-        format           => 'out%d.pnm',
         npages           => $npages,
         start            => $start,
         step             => $step,
@@ -803,8 +683,8 @@ sub scan {
             $self->signal_emit( 'finished-process', 'scan_pages' );
         },
         new_page_callback => sub {
-            my ($n) = @_;
-            $self->signal_emit( 'new-scan', $n );
+            my ( $status, $path, $n ) = @_;
+            $self->signal_emit( 'new-scan', $path, $n );
             $self->signal_emit( 'changed-progress', 0,
                 Gscan2pdf::Dialog::Scan::make_progress_string( ++$i, $npages )
             );
