@@ -24,7 +24,7 @@ Readonly my $_SANE_STATUS_EOF             => 5;      # or we could use Sane
 Readonly my $_1KB                         => 1024;
 Readonly my $ALL_PENDING_ZOMBIE_PROCESSES => -1;
 
-our $VERSION = '1.3.4';
+our $VERSION = '1.3.5';
 
 my $EMPTY = q{};
 my $COMMA = q{,};
@@ -206,7 +206,7 @@ sub _scanimage {
                                     return Glib::SOURCE_CONTINUE;
                                 }
                                 if ( defined $options{new_page_callback} ) {
-                                    $options{new_page_callback}->($id);
+                                    $options{new_page_callback}->( $path, $id );
                                 }
                                 $num_scans++;
                                 return Glib::SOURCE_REMOVE;
@@ -405,11 +405,16 @@ sub _scanadf {
                     my $timer = Glib::Timeout->add(
                         $_POLL_INTERVAL,
                         sub {
-                            if ( not -e "out$id.pnm" ) {
+                            my $path =
+                              defined( $options{dir} )
+                              ? File::Spec->catfile( $options{dir},
+                                "out$id.pnm" )
+                              : "out$id.pnm";
+                            if ( not -e $path ) {
                                 return Glib::SOURCE_CONTINUE;
                             }
                             if ( defined $options{new_page_callback} ) {
-                                $options{new_page_callback}->($id);
+                                $options{new_page_callback}->( $path, $id );
                             }
                             return Glib::SOURCE_REMOVE;
                         }
